package org.netgrok;

import com.sparsity.dex.gdb.Database;
import java.io.FileNotFoundException;
import java.io.IOException;
import org.apache.log4j.Logger;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Level;
import com.sparsity.dex.gdb.Dex;
import com.sparsity.dex.gdb.DexConfig;
import com.sparsity.dex.gdb.Graph;
import com.sparsity.dex.gdb.Session;
import java.io.File;
import java.util.concurrent.ConcurrentLinkedQueue;

public class NetGrok implements Runnable {
    
    private boolean running = true;
    private Dex instance = null;
    private Database database = null;
    private Session session = null;
    private Graph graph = null;
    final public Logger LOG = Logger.getLogger(NetGrok.class);
    final ConcurrentLinkedQueue<Source> sources = new ConcurrentLinkedQueue();
    
    public static void main(String args[]) throws IOException {
        new NetGrok(args).run();
    }
    
    public NetGrok() {
        LOG.setLevel(Level.TRACE);
        instance = new Dex(new DexConfig());
        File tempDir = null;
        
        try {
            tempDir = File.createTempFile("netgrok", null);
        } catch (IOException ex) {
            LOG.fatal("Could not create temporary directory for database!", ex);
            System.exit(-1);
        }
        
        tempDir.mkdir();
        
        try {
            database = instance.create(tempDir.toString(), "netgrok");
        } catch (FileNotFoundException ex) {
            LOG.fatal("Could not instantiate database!", ex);
            System.exit(-1);
        }
        
        session = database.newSession();
        graph = session.getGraph();
    }
    
    public NetGrok(String[] args) {
        this();
        
        if (LOG.isDebugEnabled()) {
            LOG.debug("NetGrok loaded with arguments '" + StringUtils.join(args) + "'");
        }

        // XXX - parse options here
    }
    
    public void connect(Source source) {
        LOG.info("Loading network information from " + source);
        sources.add(source);
    }
    
    public void stop() {
        this.running = false;
    }
    
    public void run() {
        while (running) {
            for (final Source source : sources) {
                if (source.isCompleted()) {
                    LOG.info(source + " has completed. Disconnecting ...");
                    sources.remove(source);
                    break;
                }
                
                try {
                    for (final Event event : source.getEvents()) {
                        if (LOG.isTraceEnabled()) {
                            LOG.trace("Processing event " + event.toString());
                        }
                        
                    }
                } catch (IOException ex) {
                    LOG.warn("Could not fetch events from " + source + ": ", ex);
                }
            }
        }
        
        LOG.info("Shutting down ...");
    }
}
